# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - xcalendar (EventAggregator)

    @copyright: 2012, 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinSupport import parseAttributes, RawParser, getPageURL
from EventAggregatorSupport.Formatting import formatEventsForOutputType, \
                                              formatEvent, formatMethod
from EventAggregatorSupport.Types import parseEventsInXMLCalendar

Dependencies = ["pages"]

# Parser support.

class Parser:

    "Interpret and show calendar information in different ways."

    Dependencies = Dependencies
    extensions = [".xcs"]

    # Input content types understood by this parser.

    input_mimetypes = ["application/calendar+xml"]

    # Output content types preferred by this parser.

    output_mimetypes = ["text/html", "application/calendar+xml"]

    def __init__(self, raw, request, **kw):

        """
        Initialise the parser with the given 'raw' data, 'request' and any
        keyword arguments that may have been supplied.
        """

        self.raw = raw
        self.request = request
        attrs = parseAttributes(kw.get("format_args", ""), False)

        self.fragment = attrs.get("fragment")

    def format(self, fmt, write=None):

        """
        Format a calendar using the given formatter 'fmt'. If the 'write'
        parameter is specified, use it to write output; otherwise, write output
        using the request.
        """

        resource = parseEventsInXMLCalendar(self.raw, getPageURL(self.request.page))

        for event in resource.getEvents():
            formatEvent(event, self.request, fmt, write=write, parser_cls=RawParser)

        formatMethod(resource, self.request, fmt, write)

    # Extra API methods.

    def formatForOutputType(self, mimetype, write=None):

        """
        Format a calendar for the given 'mimetype'. If the 'write' parameter is
        specified, use it to write output; otherwise, write output using the
        request.
        """

        # Write raw calendar information unchanged.

        if mimetype == "application/calendar+xml":
            (write or request.write)(self.raw)
        else:
            resource = parseEventsInXMLCalendar(self.raw, getPageURL(self.request.page))
            events = resource.getEvents()
            formatEventsForOutputType(events, self.request, mimetype, write=write, resource=resource)

    # Class methods.

    def getOutputTypes(self):
        return self.output_mimetypes

    getOutputTypes = classmethod(getOutputTypes)

# vim: tabstop=4 expandtab shiftwidth=4
